package com.jhelplite;

import java.awt.Component;
import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.jhelplite.JHelpLiteWindow;


/**
 * A Factory to generate JHelpLightWindow windows from a provided XML Source file.<BR>
 * <BR>
 * The factory uses the following DTD to build the help tree:<BR>
 * <BR>
 * <BR>
 *   &lt;!ELEMENT helpRoot ( helpRootName, helpRootDocument, helpRootElements ) &gt;<BR>
 * <BR>
 *   &lt;!ELEMENT helpRootName ( #PCDATA ) &gt;<BR>
 * <BR>
 *   &lt;!ELEMENT helpRootDocument ( #PCDATA ) &gt;<BR>
 * <BR>
 *   &lt;!ELEMENT helpRootElements ( helpElement | helpFolder )* &gt;<BR>
 * <BR>
 *   &lt;!ELEMENT helpElement ( helpElementName, helpElementDocument ) &gt;<BR>
 * <BR>
 *   &lt;!ELEMENT helpElementName ( #PCDATA ) &gt;<BR>
 * <BR>
 *   &lt;!ELEMENT helpElementDocument ( #PCDATA ) &gt;<BR>
 * <BR>
 *   &lt;!ELEMENT helpFolder ( helpFolderName, helpFolderDocument, helpFolderElements ) &gt;<BR>
 * <BR>
 *   &lt;!ELEMENT helpFolderName ( #PCDATA ) &gt;<BR>
 * <BR>
 *   &lt;!ELEMENT helpFolderDocument ( #PCDATA ) &gt;<BR>
 * <BR>
 *   &lt;!ELEMENT helpFolderElements ( helpElement | helpFolder )* &gt;<BR>
 * <BR>
 * <BR>
 * The JHelpLightWindow that are produced will search both on the file system and inside a package for
 * the HTML file for is to be used as the help document text.<BR>
 * <BR>
 * 
 * @version 1.0
 * @author Gerald Arnold<BR>
 * Create Date: 09/March/2007<BR>
 * Copyright: 2007<BR>
 * Company: <COMPANY NAME><BR>
 */
public final class JHelpLiteWindowFactory
{
  public static final String HELPTREEROOTNODE         = "helpRoot";
  public static final String HELPTREEROOTNAMENODE     = "helpRootName";
  public static final String HELPTREEROOTDOCUMENTNODE = "helpRootDocument";
  public static final String HELPTREEROOTELEMENTSNODE = "helpRootElements";
  public static final String HELPFOLDERNODE           = "helpFolder";
  public static final String HELPFOLDERNAMENODE       = "helpFolderName";
  public static final String HELPFOLDERDOCUMENTNODE   = "helpFolderDocument";
  public static final String HELPFOLDERELEMENTSNODE   = "helpFolderElements";
  public static final String HELPELEMENTNODE          = "helpElement";
  public static final String HELPELEMENTNAMENODE      = "helpElementName";
  public static final String HELPELEMENTDOCUMENTNODE  = "helpElementDocument";




  /**
   * Returns a JHelpLightWindow using the contents of the helpDocument to populate the tree.
   * 
   * @param helpXMLFilename  The filename of the file containing the XML structure for the help index and help page map.
   *
   * @return  JHelpLightWindow - The completed JHelpLightWindow, containing the provided index, and maps to help pages.
   * 
   * @throws ParserConfigurationException  Thrown if a DocumentBuilder cannot be created which satisfies the configuration requested
   * @throws SAXException  Thrown if the root element can not be found in the XML document.
   * @throws IOException   Thrown if there is a problem reading from the source XML file.
   */
  public static JHelpLiteWindow getInstance(String helpXMLFilename)
    throws ParserConfigurationException, SAXException, IOException
  {
    return getInstance(helpXMLFilename, null);
  }




  /**
   * Returns a JHelpLightWindow using the contents of the helpDocument to populate the tree.
   * 
   * @param helpXMLFilename  The filename of the file containing the XML structure for the help index and help page map.
   * @param parentComponent  The Parent Component of this window.
   *
   * @return  JHelpLightWindow - The completed JHelpLightWindow, containing the provided index, and maps to help pages.
   * 
   * @throws ParserConfigurationException  Thrown if a DocumentBuilder cannot be created which satisfies the configuration requested
   * @throws SAXException  Thrown if the root element can not be found in the XML document.
   * @throws IOException   Thrown if there is a problem reading from the source XML file.
   */
  public static JHelpLiteWindow getInstance(String helpXMLFilename, Component parentComponent)
    throws ParserConfigurationException, SAXException, IOException
  {
    return getInstance(helpXMLFilename, parentComponent, null);
  }




  /**
   * Returns a JHelpLightWindow using the contents of the helpDocument to populate the tree.
   * 
   * @param helpXMLFilename  The filename of the file containing the XML structure for the help index and help page map.
   * @param parentComponent  The Parent Component of this window.
   * @param windowTitle      The title for this frame.
   *
   * @return  JHelpLightWindow - The completed JHelpLightWindow, containing the provided index, and maps to help pages.
   * 
   * @throws ParserConfigurationException  Thrown if a DocumentBuilder cannot be created which satisfies the configuration requested
   * @throws SAXException  Thrown if the root element can not be found in the XML document.
   * @throws IOException   Thrown if there is a problem reading from the source XML file.
   */
  public static JHelpLiteWindow getInstance(String helpXMLFilename, Component parentComponent, String windowTitle)
    throws ParserConfigurationException, SAXException, IOException
  {
    File helpXMLFile = new File(helpXMLFilename);
    return getInstance(helpXMLFile, parentComponent, windowTitle);
  }




  /**
   * Returns a JHelpLightWindow using the contents of the helpDocument to populate the tree.
   * 
   * @param helpXMLFile  The file containing the XML structure for the help index and help page map.
   *
   * @return  JHelpLightWindow - The completed JHelpLightWindow, containing the provided index, and maps to help pages.
   * 
   * @throws ParserConfigurationException  Thrown if a DocumentBuilder cannot be created which satisfies the configuration requested
   * @throws SAXException  Thrown if the root element can not be found in the XML document.
   * @throws IOException   Thrown if there is a problem reading from the source XML file.
   */
  public static JHelpLiteWindow getInstance(File helpXMLFile)
    throws ParserConfigurationException, SAXException, IOException
  {
    return getInstance(helpXMLFile, null);
  }




  /**
   * Returns a JHelpLightWindow using the contents of the helpDocument to populate the tree.
   * 
   * @param helpXMLFile      The file containing the XML structure for the help index and help page map.
   * @param parentComponent  The Parent Component of this window.
   *
   * @return  JHelpLightWindow - The completed JHelpLightWindow, containing the provided index, and maps to help pages.
   * 
   * @throws ParserConfigurationException  Thrown if a DocumentBuilder cannot be created which satisfies the configuration requested
   * @throws SAXException  Thrown if the root element can not be found in the XML document.
   * @throws IOException   Thrown if there is a problem reading from the source XML file.
   */
  public static JHelpLiteWindow getInstance(File helpXMLFile, Component parentComponent)
    throws ParserConfigurationException, SAXException, IOException
  {
    return getInstance(helpXMLFile, parentComponent, null);
  }




  /**
   * Returns a JHelpLightWindow using the contents of the helpDocument to populate the tree.
   * 
   * @param helpXMLFile      The file containing the XML structure for the help index and help page map.
   * @param parentComponent  The Parent Component of this window.
   * @param windowTitle      The title for this frame.
   *
   * @return  JHelpLightWindow - The completed JHelpLightWindow, containing the provided index, and maps to help pages.
   * 
   * @throws ParserConfigurationException  Thrown if a DocumentBuilder cannot be created which satisfies the configuration requested
   * @throws SAXException  Thrown if the root element can not be found in the XML document.
   * @throws IOException   Thrown if there is a problem reading from the source XML file.
   */
  public static JHelpLiteWindow getInstance(File helpXMLFile, Component parentComponent, String windowTitle)
    throws ParserConfigurationException, SAXException, IOException
  {
    if (!helpXMLFile.exists())
      throw new IOException("XML Source file does not exist.");

    if (!helpXMLFile.isFile())
      throw new IOException("XML Source file is not a file.");

    if (!helpXMLFile.canRead())
      throw new IOException("XML Source file can not be read.");

    DocumentBuilder documentBuilder;

    Document helpDocument = null;

    // Setup the environment ready to load the XML config file
    documentBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();

    helpDocument = documentBuilder.parse(helpXMLFile);
    helpDocument.getDocumentElement().normalize();

    return getInstance(helpDocument, parentComponent, windowTitle);
  }




  /**
   * Returns a JHelpLightWindow using the contents of the helpDocument to populate the tree.
   * 
   * @param helpDocument  The Document containing the XML structure for the help index and help page map.
   * 
   * @return  JHelpLightWindow - The completed JHelpLightWindow, containing the provided index, and maps to help pages.
   * 
   * @throws SAXException  Thrown if the root element can not be found in the XML document.
   */
  public static JHelpLiteWindow getInstance(Document helpDocument)
    throws SAXException
  {
    return getInstance(helpDocument, null);
  }




  /**
   * Returns a JHelpLightWindow using the contents of the helpDocument to populate the tree.
   * 
   * @param helpDocument     The Document containing the XML structure for the help index and help page map.
   * @param parentComponent  The Parent Component of this window.
   * 
   * @return  JHelpLightWindow - The completed JHelpLightWindow, containing the provided index, and maps to help pages.
   * 
   * @throws SAXException  Thrown if the root element can not be found in the XML document.
   */
  public static JHelpLiteWindow getInstance(Document helpDocument, Component parentComponent)
    throws SAXException
  {
    return getInstance(helpDocument, parentComponent, null);
  }




  /**
   * Returns a JHelpLightWindow using the contents of the helpDocument to populate the tree.
   * 
   * @param helpDocument     The Document containing the XML structure for the help index and help page map.
   * @param parentComponent  The Parent Component of this window.
   * @param windowTitle      The title for this frame.
   * 
   * @return  JHelpLightWindow - The completed JHelpLightWindow, containing the provided index, and maps to help pages.
   * 
   * @throws SAXException  Thrown if the root element can not be found in the XML document.
   */
  public static JHelpLiteWindow getInstance(Document helpDocument, Component parentComponent, String windowTitle)
    throws SAXException
  {
    // Create the initial window object
    JHelpLiteWindow helpWindow;
    if (parentComponent == null)
      helpWindow = new JHelpLiteWindow();

    else
      helpWindow = new JHelpLiteWindow(parentComponent);

    if (windowTitle != null && windowTitle.trim().length() > 0)
      helpWindow.setFrameTitle(windowTitle);

    // Get the root element from the XML file
    NodeList rootNodeList = helpDocument.getElementsByTagName(HELPTREEROOTNODE);




    /* ****************************************************
     *   Lets deal with the document root to begin with   *
     ******************************************************/
    @SuppressWarnings("unchecked")
    Vector helpContents = new Vector();
    if (rootNodeList.getLength() > 0)
    {
      // Load the root node
      Node rootNode = rootNodeList.item(0);

      if (rootNode.getNodeType() == Node.ELEMENT_NODE)
      {
        Element rootElement = (Element) rootNode;

        // Now load the rootNode Name
        String rootNodeName     = "Help";
        NodeList rootNodeNameList = rootElement.getElementsByTagName(HELPTREEROOTNAMENODE);
        if (rootNodeNameList.getLength() > 0)
        {
          // Load the root node name
          Node node = rootNodeNameList.item(0);

          if (node.getNodeType() == Node.ELEMENT_NODE &&
              node.getNodeName().equals(HELPTREEROOTNAMENODE))
          {
            Element  element  = (Element) node;
            NodeList nodeList = element.getChildNodes();
            rootNodeName = nodeList.item(0).getNodeValue().trim();
          }
        }

        // Load the rootNode document
        String rootNodeDocument = "";
        rootNodeNameList = rootElement.getElementsByTagName(HELPTREEROOTDOCUMENTNODE);
        if (rootNodeNameList.getLength() > 0)
        {
          // Load the root node document name
          Node node = rootNodeNameList.item(0);

          // Load the root document name
          if (node.getNodeType() == Node.ELEMENT_NODE &&
              node.getNodeName().equals(HELPTREEROOTDOCUMENTNODE))
          {
            Element  element  = (Element) node;
            NodeList nodeList = element.getChildNodes();
            rootNodeDocument = nodeList.item(0).getNodeValue().trim();
          }
        }

        JHelpLiteUserObject rootNodeObject = new JHelpLiteUserObject();
        rootNodeObject.setNodeName(rootNodeName);

        helpWindow.init(rootNodeObject);

        if (rootNodeDocument.length() > 0)
        {
          rootNodeObject.setNodeDocumentLocation(rootNodeDocument);
          helpWindow.setRootNodeDocumentPath(rootNodeName, rootNodeDocument);
        }



        // Now load the Help Elements
        rootNodeNameList = rootElement.getElementsByTagName(HELPTREEROOTELEMENTSNODE);
        if (rootNodeNameList.getLength() > 0)
        {
          // Load the root node document name
          Node node = rootNodeNameList.item(0);
          if (node.getNodeType() == Node.ELEMENT_NODE &&
              node.getNodeName().equals(HELPTREEROOTELEMENTSNODE))
          {
            Element  element  = (Element) node;
            NodeList nodeList = element.getChildNodes();

            helpContents = getHelpContents(nodeList, rootNodeObject, helpContents);
          }
        }
      }
    }

    else
      throw new SAXException("Missing root element");

    helpWindow.create();
    helpWindow.populateHelpIndex(helpContents);

    return helpWindow;
  }




  /**
   * Get the help document elements from the XML file
   * 
   * @param nodeList      The NodeList containing the help tree element and and associated document.
   * @param helpContents  The Vector to put the document and element names in. 
   * 
   * @return Vector - Filled with Vectors and JHelpLightUserObjects. The JHelpLightUserObjects contain the display name
   *                  the location of the file that is to be used for the help page.
   */ 
  @SuppressWarnings("unchecked")
  private static Vector getHelpContents(NodeList nodeList, JHelpLiteUserObject pathRootUserObject, Vector helpContents)
  {
    // Make sure that there are children nodes
    if (nodeList.getLength() > 0)
    {
      // Go through each of the nodes
      for (int i = 0; i < nodeList.getLength(); i++)
      {
        // Load the node document name
        Node node = nodeList.item(i);

        // Make sure it's an element node
        if (node.getNodeType() == Node.ELEMENT_NODE)
        {

          // For folders
          if (node.getNodeName().equals(HELPFOLDERNODE))
          {
            JHelpLiteUserObject folderUserObject = new JHelpLiteUserObject();

            // Load the folder name
            Element  folderElement  = (Element) node;
            NodeList folderNodeList = folderElement.getElementsByTagName(HELPFOLDERNAMENODE);
            if (folderNodeList.getLength() > 0)
            {
              // Load the node document name
              Node folderNameNode = folderNodeList.item(0);
              if (folderNameNode.getNodeType() == Node.ELEMENT_NODE &&
                  folderNameNode.getNodeName().equals(HELPFOLDERNAMENODE))
              {
                Element  folderNameElement = (Element) folderNameNode;
                NodeList folderNameElementnodeList = folderNameElement.getChildNodes();

                // Set the name of this item
                folderUserObject.setNodeName(folderNameElementnodeList.item(0).getNodeValue().trim());
              }
            }


            // Now load the folder document location
            folderNodeList = folderElement.getElementsByTagName(HELPFOLDERDOCUMENTNODE);
            if (folderNodeList.getLength() > 0)
            {
              // Load the node document name
              Node folderDocoNode = folderNodeList.item(0);
              if (folderDocoNode.getNodeType() == Node.ELEMENT_NODE &&
                  folderDocoNode.getNodeName().equals(HELPFOLDERDOCUMENTNODE))
              {
                Element  folderDocoElement = (Element) folderDocoNode;
                NodeList folderDocoElementnodeList = folderDocoElement.getChildNodes();

                // Set the name of this item
                if (folderDocoElementnodeList.item(0) != null)
                  folderUserObject.setNodeDocumentLocation(folderDocoElementnodeList.item(0).getNodeValue().trim());
              }
            }

            // Now add this object to the vector of objects
            helpContents.add(folderUserObject);



            // Now load the Folder Elements
            folderNodeList = folderElement.getElementsByTagName(HELPFOLDERELEMENTSNODE);
            if (folderNodeList.getLength() > 0)
            {
              // Load the root node document name
              Node folderElementsNode = folderNodeList.item(0);
              if (folderElementsNode.getNodeType() == Node.ELEMENT_NODE &&
                  folderElementsNode.getNodeName().equals(HELPFOLDERELEMENTSNODE))
              {
                Element  folderElementsElement = (Element) folderElementsNode;
                NodeList folderElementsNodeList = folderElementsElement.getChildNodes();

                helpContents.add(getHelpContents(folderElementsNodeList, folderUserObject, new Vector()));
              }
            }
          }



          // For the leaf nodes
          else if (node.getNodeName().equals(HELPELEMENTNODE))
          {
            JHelpLiteUserObject leafUserObject = new JHelpLiteUserObject();

            // Load the leaf name
            Element  leafElement  = (Element) node;
            NodeList leafNodeList = leafElement.getElementsByTagName(HELPELEMENTNAMENODE);
            if (leafNodeList.getLength() > 0)
            {
              // Load the node document name
              Node leafNameNode = leafNodeList.item(0);
              if (leafNameNode.getNodeType() == Node.ELEMENT_NODE &&
                  leafNameNode.getNodeName().equals(HELPELEMENTNAMENODE))
              {
                Element  leafNameElement = (Element) leafNameNode;
                NodeList leafNameElementnodeList = leafNameElement.getChildNodes();

                // Set the name of this item
                leafUserObject.setNodeName(leafNameElementnodeList.item(0).getNodeValue().trim());
              }
            }

            leafNodeList = leafElement.getElementsByTagName(HELPELEMENTDOCUMENTNODE);
            if (leafNodeList.getLength() > 0)
            {
              // Load the node document
              Node leafDocumentNode = leafNodeList.item(0);
              if (leafDocumentNode.getNodeType() == Node.ELEMENT_NODE &&
                  leafDocumentNode.getNodeName().equals(HELPELEMENTDOCUMENTNODE))
              {
                Element  leafDocumentElement = (Element) leafDocumentNode;
                NodeList leafDocumentElementnodeList = leafDocumentElement.getChildNodes();

                // Set the name of this item
                leafUserObject.setNodeDocumentLocation(leafDocumentElementnodeList.item(0).getNodeValue().trim());
              }
            }

            // Now add this object to the vector of objects
            helpContents.add(leafUserObject);
          }
        }
      }
    }

    return helpContents;
  }
}